utility MaterialMaster "MatMaster"
(
--------------------------------------------------
--MaterialMaster
--Version 0.1 Beta!
--Started: 3/23/2000
--Edited:  3/29/2000
--by Borislav Petrov
--http://gfxcentral.com/bobo/
--------------------------------------------------
--Developed for 
--Charles Ernst,
--Midway Games
--------------------------------------------------


local valid_array = #()
local class_array = #()
local filter_prop_names = #()
local default_values = #()

local preset_files_list = #()

local preset_values = #()
local preset_valid_array = #()
local preset_class_array = #()
local preset_prop_names = #()

local preset_file_name 
local current_preset_name

fn collect_material_properties temp_mat =
(
prop_names = getpropnames temp_mat

for i = 1 to prop_names.count do
(
try
(
check_class = classof (getproperty temp_mat prop_names[i]) 
)
catch
(
check_class = UndefinedClass
)
if check_class != UndefinedClass and check_class != ArrayParameter and check_class != String and check_class != SubAnim then
	(
	if findItem filter_prop_names (prop_names[i] as string) == 0 then
		(
		append valid_array i
		append class_array check_class 
		append filter_prop_names (prop_names[i] as string)
		append default_values (getproperty temp_mat prop_names[i]) 
		)
	)
)
)--end fn

button update_materials "UPDATE MATERIALS" width:150 align:#center height:35
group "Affect Materials in:"
(
radiobuttons update_mode labels:#("Scene","Selection")
)

edittext edit_name width:155 align:#center offset:[-2,0]
listbox presets_list  items:#() width:155 align:#center height:4 
button create_preset "Create " width:45 align:#left across:3 offset:[-5,0] tooltip:"Create New empty Preset"
button clone_preset "Clone " width:48 align:#center tooltip:"Clone Current Preset" enabled:false offset:[-2,0]
button delete_preset "Delete" width:45 align:#right tooltip:"Delete Preset from disk" enabled:false

dropdownlist mat_type items:#("Standard Material","Raytrace Material","UserDefined - MEdit Slot #1") width:155 align:#center 

combobox prop_names_list "Available Material Properties:" items:#() width:155 align:#center 
button add_to_list "Add... " width:70 align:#left across:2 offset:[-5,0] tooltip:"Add To Preset List" enabled:false
button remove_from_list "Remove..." width:70 align:#right tooltip:"Remove From Preset List" enabled:false
listbox preset_names_list "Preset Properties:" items:#() width:155 align:#center height:5
group "Preset Value:"
(
pickbutton get_from_scene ">Get From Scene Object" align:#right width:135
spinner spinner_float "Float:" align:#right fieldwidth:52 enabled:false range:[-1000,1000,0]
spinner spinner_integer "Integer:" align:#right fieldwidth:52 enabled:false type:#integer range:[-1000,1000,0]
colorpicker color_value "Color:" align:#right fieldwidth:60 enabled:false
checkbutton binary_value "Binary Value" align:#right width:135 enabled:false highlightcolor:(color 200 255 200)
)

on mat_type selected itm do
(
valid_array = #()
class_array = #()
filter_prop_names = #()
default_values = #()
case itm of
(
1: temp_mat = standardmaterial () 
2: temp_mat = raytracematerial ()
3: temp_mat = meditmaterials[1]
)
collect_material_properties temp_mat 
prop_names_list.items = filter_prop_names
)


fn update_switches =
(
	try(
		spinner_float.enabled = preset_class_array[preset_names_list.selection] == Float
		spinner_integer.enabled = preset_class_array[preset_names_list.selection] == Integer
		binary_value.enabled = preset_class_array[preset_names_list.selection] == BooleanClass
		color_value.enabled = preset_class_array[preset_names_list.selection] == Color
		
		if spinner_float.enabled then spinner_float.value = preset_values[preset_names_list.selection]
		if spinner_integer.enabled then spinner_integer.value = preset_values[preset_names_list.selection]
		if binary_value.enabled then binary_value.checked = preset_values[preset_names_list.selection]
		if color_value.enabled then color_value.color = preset_values[preset_names_list.selection]
	)
	catch
	(
		spinner_float.enabled = false
		spinner_integer.enabled = false
		binary_value.enabled = false
		color_value.enabled = false
	)	
)

fn save_preset_to_disk f_name=
(
makedir (scriptspath+"\\matmaster\\")
out_file = createfile f_name
format "[MatMaster Presets File]\n" to:out_file
format "%\n" current_preset_name to:out_file
format "%\n" preset_prop_names.count to:out_file

for i = 1 to preset_prop_names.count do
	(
	format "%\n" preset_prop_names[i] to:out_file
	format "%\n" (preset_class_array[i] as string) to:out_file
	if preset_class_array[i] == Color then
		(
		format "%\n" ((preset_values[i] as point3) as string) to:out_file
		)
		else
		(
		format "%\n" (preset_values[i] as string) to:out_file
		)
	)
format "[END File]\n" to:out_file	
close out_file
)

fn load_preset_from_disk f_name=
(
preset_values = #()
preset_class_array = #()
preset_prop_names = #()

makedir (scriptspath+"\\matmaster\\")
in_file = openfile f_name
str = readline in_file 
if str == "[MatMaster Presets File]" then
(
	str = readline in_file 
	current_preset_name = str
	str = readline in_file 
	read_counter = str as integer

end_of_file = false
for i = 1 to read_counter do
	(
	str = readline in_file 
	append preset_prop_names str
	str = readline in_file 
	append preset_class_array (execute str)
	str = readline in_file 
	append preset_values (execute str)
	)--end i loop
)--end if 
close in_file
preset_names_list.items = preset_prop_names 
if preset_names_list.selection > preset_names_list.items.count then preset_names_list.selection = preset_names_list.items.count
update_switches()
)--end fn


fn set_enabled =
(
delete_preset.enabled = presets_list.items.count > 0
clone_preset.enabled = presets_list.items.count > 0

add_to_list.enabled = presets_list.selection > 0
if presets_list.items.count == 0 then add_to_list.enabled = false
remove_from_list.enabled = preset_names_list.items.count > 0 
)


on clone_preset pressed do
(
preset_file_name = scriptspath+"\\matmaster\\MM_"+((random 0 10000)as string)+"_"+((random 0 10000)as string)+"_"+((random 0 10000)as string)+".MMP"
append preset_files_list preset_file_name
temp_array = presets_list.items
current_preset_name += "_Clone"
append temp_array current_preset_name 
presets_list.items = temp_array 
save_preset_to_disk preset_file_name 
presets_list.selection = presets_list.items.count
)

on create_preset pressed do
(
current_preset_name = "Preset "+(preset_files_list.count+1) as string
preset_file_name = scriptspath+"\\matmaster\\MM_"+((random 0 10000)as string)+"_"+((random 0 10000)as string)+"_"+((random 0 10000)as string)+".MMP"
append preset_files_list preset_file_name 
save_preset_to_disk preset_file_name 
temp_array = presets_list.items
append temp_array current_preset_name 
presets_list.items = temp_array 
delete_preset.enabled = true
add_to_list.enabled = true 
presets_list.selection = presets_list.items.count 
)

on delete_preset pressed do
(
if presets_list.selection > 0 then
	(
	q = querybox "About Deleting the current Preset - No Undo!" title:"MatMaster"
	if q then
		(
		deletefile preset_files_list[presets_list.selection]
		deleteItem preset_files_list presets_list.selection
		temp_array = presets_list.items
		deleteItem  temp_array presets_list.selection
		presets_list.items = temp_array 
		preset_values = #()
		preset_class_array = #()
		preset_prop_names = #()
		preset_names_list.items = preset_prop_names
		if presets_list.selection > 1 then presets_list.selection -= 1 
		if presets_list.selection == undefined then presets_list.selection = presets_list.items.count 
		if presets_list.selection == 0 then presets_list.selection = presets_list.items.count 
		
		set_enabled ()		
		)
	)
)

on edit_name entered nam do
(
if nam != "" then
	(
	temp_array = presets_list.items
	temp_array[presets_list.selection] = nam
	presets_list.items = temp_array 
	current_preset_name = nam
	save_preset_to_disk preset_files_list[presets_list.selection]
	)
)




on update_materials pressed do
(
if update_mode.state == 2 then 
	(
	mat_array = #()
	for i in selection do if i.material!=undefined then 
		(
		append mat_array i.material
		if classof i.material == MultiMaterial then
		(
		for j = 1 to i.material.numsubs do
			(
			append mat_array (i.material[j])
			)
		)
		else
		(
		mat_subs = getSubAnimNames 	i.material
		for s in mat_subs do
			(
			try
			(
			if superclassof (getProperty i.material s) == material then
				(
				append mat_array (getProperty i.material s)
				)
			)catch()	
			)--end s loop
		)	
		)--end i loop
	)
	else
	(
	mat_array = #()
	for i in geometry do if i.material!=undefined then 
		(
		append mat_array i.material
		if classof i.material == MultiMaterial then
		(
		for j = 1 to i.material.numsubs do
			(
			append mat_array (i.material[j])
			)
		)
		else
		(
		mat_subs = getSubAnimNames 	i.material
		for s in mat_subs do
			(
			try
			(
			if superclassof (getProperty i.material s) == material then
				(
				append mat_array (getProperty i.material s)
				)
			)catch()	
			)--end s loop
		)--end if 	
		)
	)
for m in mat_array do
(
for i = 1 to preset_prop_names.count do
	(
	try
	(
	setproperty m preset_prop_names[i] preset_values[i]
	)
	catch()
	)--end i loop
)--end m loop
)


on preset_names_list selected itm do
(
update_switches()
)

on get_from_scene picked obj do
(
if obj.material != undefined then
	(
for i = 1 to preset_prop_names.count do
	(
	try
	(
	preset_values[i] = getproperty obj.material preset_prop_names[i] 
	)
	catch()
	)--end i loop
	update_switches()
	)
)


fn add_to_presets =
(
if findItem preset_prop_names prop_names_list.items[prop_names_list.selection] == 0 then 
	(
	append preset_prop_names prop_names_list.items[prop_names_list.selection]
	append preset_class_array class_array[prop_names_list.selection]
	append preset_values default_values[prop_names_list.selection]
	)
save_preset_to_disk preset_files_list[presets_list.selection]
preset_names_list.items = preset_prop_names 
preset_names_list.selection = preset_names_list.items.count
update_switches()
set_enabled ()
if prop_names_list.selection < prop_names_list.items.count then prop_names_list.selection += 1
)

on add_to_list pressed do
(
add_to_presets ()
)

on remove_from_list pressed do
(
pnl = preset_names_list.selection
if pnl > 0 then
(
	deleteItem preset_prop_names pnl 
	deleteItem preset_class_array pnl 
	deleteItem preset_values pnl 
	save_preset_to_disk preset_files_list[presets_list.selection]
	preset_names_list.items = preset_prop_names 
	if preset_names_list.selection > preset_names_list.items.count then preset_names_list.selection = preset_names_list.items.count
	update_switches()
	set_enabled ()
)
	
)

on prop_names_list doubleclicked itm do
(
if presets_list.items.count > 0 then add_to_presets ()
)

on presets_list selected itm do 
(
load_preset_from_disk preset_files_list[presets_list.selection]
set_enabled ()
edit_name.text = current_preset_name 
)


on spinner_float changed val do 
	(
	try
	(
	preset_values[preset_names_list.selection] = val
	save_preset_to_disk preset_files_list[presets_list.selection]
	)catch()
	)

on spinner_integer changed val do 
	(
	try
	(
	preset_values[preset_names_list.selection] = val
	save_preset_to_disk preset_files_list[presets_list.selection]
	)catch()
	)

on binary_value changed val do 
	(
	try
	(
	preset_values[preset_names_list.selection] = val
	save_preset_to_disk preset_files_list[presets_list.selection]
	)catch()
	)

on color_value changed val do 
	(
	try
	(
	preset_values[preset_names_list.selection] = val
	save_preset_to_disk preset_files_list[presets_list.selection]
	)catch()
	)


on MaterialMaster open do
(
valid_array = #()
class_array = #()
filter_prop_names = #()
default_values = #()

temp_mat = standardmaterial ()
collect_material_properties temp_mat 
prop_names_list.items = filter_prop_names
preset_files_list = getFiles (scriptspath+"\\matmaster\\*.MMP")
temp_array = #()
for i in preset_files_list do
	(
	in_file = openfile i
	str = readline in_file
	if str == "[MatMaster Presets File]" then
		(
		str = readline in_file
		append temp_array str
		close in_file
		)--end if
	)--end i loop
	presets_list.items = temp_array
	if presets_list.items.count > 0 then
		(
		presets_list.selection = 1
		load_preset_from_disk preset_files_list[1]
		)
	set_enabled ()
)--end open

)--end util